﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.CommandWpf;

namespace XamDataTreeEx
{
    public class CompareBillingVM : ViewModelBase
    {
        #region Constructor

        /// <summary>
        /// Ctor
        /// </summary>
        /// <param name="billingCycleID">Billing Cycle ID</param>
        /// <param name=billingCyclePhase>Billing Cycle Phase</param>
        /// <param name="billingModelID">Billing Model ID</param>
        /// <param name="computerName">Computer Machine Name</param>
        public CompareBillingVM(int? billingCycleID, int? billingCyclePhase, int? billingModelID, string computerName)
        {
            BillingCycleID = billingCycleID;
            BillingCyclePhase = billingCyclePhase;
            BillingModelID = billingModelID;
            ComputerName = computerName;

            Setup();
            LoadAsWindow();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Billing Cycle ID
        /// </summary>
        private int? BillingCycleID { get; set; }

        /// <summary>
        /// Billing Cycle Phase
        /// </summary>
        private int? BillingCyclePhase { get; set; }

        /// <summary>
        /// Billing Model ID
        /// </summary>
        private int? BillingModelID { get; set; }

        /// <summary>
        /// Computer Machine Name
        /// </summary>
        private string ComputerName { get; set; }

        /// <summary>
        /// View Model instance of exception.  If there is an error loading, this will be set to not null exception (workaround for Loaded event bug that does not rethrow exception correctly)
        /// </summary>
        public Exception Exception;

        private bool _isDirty;
        /// <summary>
        /// Bool for whether the collection of Adjustment Reasons has changed (whether items added or existing modified)
        /// </summary>
        public bool IsDirty
        {
            get { return _isDirty; }
            set
            {
                if (_isDirty != value)
                {
                    _isDirty = value;
                    RaisePropertyChanged(() => IsDirty);
                }
            }
        }

        private bool? _dialogResult;
        /// <summary>
        /// Property true to close window from ViewModel-must me of "GenericDialog" type
        /// </summary>
        public bool? DialogResult
        {
            get { return _dialogResult; }
            set
            {
                if (_dialogResult != value)
                {
                    _dialogResult = value;
                    RaisePropertyChanged(() => DialogResult);
                }
            }
        }

        /// <summary>
        /// The Apply button has been clicked
        /// </summary>
        public bool IsApply;

        /// <summary>
        /// The collection of compare billing results from the stored procedure
        /// </summary>
        public ObservableCollection<CompareBilling> AllCompares
        {
            get;
            private set;
        }

        /// <summary>
        /// Items
        /// </summary>
        public ObservableCollection<Item> AllItems
        {
            get;
            private set;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Set up the view model 
        /// </summary>
        private void Setup()
        {
            AddCommands();

            AllCompares = new ObservableCollection<CompareBilling>();
            AllItems = new ObservableCollection<Item>();
            AllItems.CollectionChanged += AllItems_CollectionChanged;
        }

        /// <summary>
        /// Collection changed event handler for All Items
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AllItems_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.NewItems != null)
            {
                foreach (Item item in e.NewItems)
                {
                    item.PropertyChanged += Item_PropertyChanged;
                }
            }
            if (e.OldItems != null)
            {
                foreach (Item item in e.OldItems)
                {
                    item.PropertyChanged -= Item_PropertyChanged;
                }
            }
        }

        /// <summary>
        /// Item property changed event handler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            IsDirty = true;
        }

        /// <summary>
        /// Add all commands
        /// </summary>
        private void AddCommands()
        {
            ApplyCommand = new RelayCommand(() => ApplyCommandExecuted());
            CloseCommand = new RelayCommand(() => CloseCommandExecuted());
            WindowClosingCommand = new RelayCommand<CancelEventArgs>((args) => ConfirmClose(args));
        }

        Item root = new Item("All Billing Types", false, false);

        /// <summary>
        /// Setup the About window
        /// </summary>
        private void LoadAsWindow()
        {
            try
            {
                ClearAll();

                AllCompares = new ObservableCollection<CompareBilling>();
                AllItems = new ObservableCollection<Item>();

                App.Current.Dispatcher.Invoke((Action)delegate()
                    {
                        AllCompares.Add(new CompareBilling("xxx", "yy", false, true, true, 1, true, "dd", "D", "Comparison"));
                    });

                Item item = new Item(AllCompares[0]);

                root.Children.Add(item);

                App.Current.Dispatcher.Invoke((Action)delegate()
                {
                    root.Initialize();
                    AllItems.Add(root);
                });
                IsDirty = true;
            }
            catch (Exception ex)
            {
                IsDirty = false;
                Exception = ex;
            }
        }

        /// <summary>
        /// Method removes last section of KeyData to find the Parent KeyData
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        private string DropKeyEnd(string key)
        {
            string[] keyParts = key.Split('.');
            string result;

            //special code for subvendors
            if (keyParts.Length >= 6 && keyParts[2] == keyParts[4] && keyParts[3] == "4")  //4=subvendor
            {
                var keyArray = keyParts.Take(3);
                result = String.Join(".", keyArray);
            }
            else
            {
                var keyArray = keyParts.Take(keyParts.Length - 1);
                result = String.Join(".", keyArray);
            }

            return result;
        }

        /// <summary>
        /// Attempts to add Item to the correct collection of children in its parent (recursive)
        /// </summary>
        /// <param name="target"></param>
        /// <param name="item"></param>
        /// <param name="parentKeyData"></param>
        private void AddChildToParent(Item target, Item item, string parentKeyData)
        {
            foreach (Item node in target.Children)
            {
                if (String.Equals(node.KeyData, parentKeyData))
                {
                    App.Current.Dispatcher.Invoke((Action)delegate()
                    {
                        node.Children.Add(item);
                    });
                }
                else
                {
                    App.Current.Dispatcher.Invoke((Action)delegate()
                    {
                        AddChildToParent(node, item, parentKeyData);
                    });
                }
            }
        }

        /// <summary>
        /// Clear all data
        /// </summary>
        private void ClearAll()
        {
            App.Current.Dispatcher.Invoke((Action)delegate()
            {
                AllCompares.Clear();
                AllItems.Clear();
                IsDirty = false;
            });
        }

        /// <summary>
        /// Applies changes
        /// </summary>
        private void ApplyCommandExecuted()
        {
            try
            {
                foreach (Item item in AllItems)
                {
                    UpdateIfIsAcceptableDifference(item);
                }

                IsDirty = false;
                IsApply = true;
                DialogResult = true;
                
            }
            catch (Exception ex)
            {
                IsDirty = false;
                DialogResult = true;
                IsApply = true;
                
                Exception = ex;
            }
        }

        private void UpdateIfIsAcceptableDifference(Item node)
        {
            if (node.Children.Count == 0)
            {
                if (node.IsDirty)
                {
                    // CompareBillingData.Instance.UpdateDifferenceFlag(node.DifferencesID, (bool)node.IsChecked);
                }
            }
            else
            {
                foreach (Item child in node.Children)
                {
                    UpdateIfIsAcceptableDifference(child);
                }
            }
        }

        /// <summary>
        /// Close this view
        /// </summary>
        private void CloseCommandExecuted()
        {
            //ClearAll();
            DialogResult = true;
        }

        /// <summary>
        /// Asks to confirm closing of window, whether from 'X' or Apply or Close 
        /// </summary>
        /// <param name="e"></param>
        private void ConfirmClose(CancelEventArgs e)
        {
            //if (!IsApply)
            {
               // if (IsDirty)
                {
                    ClearAll();                //this ClearAll method is the offender, commenting it out fixes the problem
                    IsDirty = false;
                    e.Cancel = false;
                }
                //else
                //{
                //    e.Cancel = true;
                //    DialogResult = false;
                //}
            }
        }

        #endregion

        #region Commands

        /// <summary>
        /// The Apply Command
        /// </summary>
        public RelayCommand ApplyCommand
        {
            get;
            private set;
        }

        /// <summary>
        /// The Close command
        /// </summary>
        public RelayCommand CloseCommand
        {
            get;
            private set;
        }

        /// <summary>
        /// Command will allow us to handle window closing event
        /// </summary>
        public RelayCommand<CancelEventArgs> WindowClosingCommand
        {
            get;
            private set;
        }

        #endregion
    }
}
